#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <stdarg.h>
#include <limits.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "config.h"
#include "imapfilter.h"
#include "log.h"


extern char logfile[PATH_MAX];
extern unsigned int options;

static FILE *logfp = NULL;	/* Points to logfile. */


/*
 * Prints message if not in OPT_DETAILS_QUITE mode.
 */
void info(const char *info, ...)
{
    va_list args;

    if (!(options & OPT_DETAILS_QUITE)) {
	va_start(args, info);
	vprintf(info, args);
	va_end(args);
    }
}


/*
 * Prints message if in OPT_DETAILS_VERBOSE mode.
 */
void verbose(const char *info, ...)
{
    va_list args;

    if (options & OPT_DETAILS_VERBOSE) {
	va_start(args, info);
	vprintf(info, args);
	va_end(args);
    }
}

/*
 * Prints error message and writes it into logfile.
 */
void error(const char *errmsg, ...)
{
    va_list args;

    va_start(args, errmsg);

    if (logfp) {
	vfprintf(logfp, errmsg, args);
    }

    vfprintf(stderr, errmsg, args);
    va_end(args);
}


/*
 * Opens the file to save logging information.
 */
int open_logfile(void)
{
    if (!(logfile[0]))
	return 0;		/* Logging not enabled. */

#ifdef DEBUG
    printf("debug: logfile: '%s'\n", logfile);
#endif

    if (create_logfile())
	return 1;

    logfp = fopen(logfile, "a");

    if (!logfp) {
	fprintf(stderr, "imapfilter: opening logfile %s: %s\n",
		logfile, strerror(errno));
	return 1;
    }

    return 0;
}


/*
 *
 */
int create_logfile(void)
{
    int fd;
    struct stat fs;

    if (stat(logfile, &fs) && errno == ENOENT) {
	fd = creat(logfile, S_IRUSR | S_IWUSR);
	close(fd);
	return 0;
    } else if (!S_ISREG(fs.st_mode)) {
	printf
	    ("imapfilter: file %s already exists and not a regular file\n",
	     logfile);
	return 1;
    }

    return 0;
}


/*
 * Closes the logfile.
 */
int close_logfile(void)
{
    if (!logfp)
	return 0;
    else
	return fclose(logfp);
}


/*
 * Writes information to logfile.
 */
void log_info(const char *info, ...)
{
    va_list args;

    if (logfp) {
	va_start(args, info);
	vfprintf(logfp, info, args);
	va_end(args);
    }
}
